/*
  Copyright (c) 2019, WOZ, woz <at> pin4 <dot> at
  All rights reserved.

  *********** animated PETSCII X-mas star ***********

  Based on an animated GIF shared by Logiker in VCC chat.
  To be used with CC65 for the C64 platform.
  Proof of concept -> no check for boundaries if star gets too big(!)

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
      * Neither the name of the author(s) nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.
      * Commercial use prohibited without permission of the author(s).

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdlib.h>
#include <string.h>
#include <cbm.h>

/* just a quick hack to test double-buffering using VIC bank switching */
unsigned int screenstart=0x400U;

/* clear screen */
void cls() {
    memset((void *)screenstart,32,1000); // Screen
}

/* set specified text colour */
void col(char col) {
    memset((void *)55296U,col,1000); // Colour
}

/* draw a horizontal petscii line with inverted start/end chars */
void hline(char x, char y, char l, char c) {
  char *p = (char *)(screenstart+x+y*40);
  char i;
  if (l<2) return;
  l-=2;
  *p = c;
  ++p;
  for(i=0;i<l;++i) {
    *p = 32|128;
    ++p;
  }
  *p = c^128;
}

/* draw a vertical petscii line with inverted start/end chars */
void vline(char x, char y, char l, char c) {
  char *p = (char *)(screenstart+x+y*40);
  char i;
  if (l<2) return;
  l-=2;
  *p = c;
  p+=40;
  for(i=0;i<l;++i) {
    *p = 32|128;
    p+=40;
  }
  *p = c^128;
}

/* draw a complete star with given size */
void star(char x, char y, char l) {
  char i;

  /*          *          */
  *(char *)(screenstart+x+y*40) = 81; //90;

  if (l<1) return;
  
  for (i=0;i<l;++i) {
    char l2=l<<1;
   /*           /|       *
     *         | |       *
     *         |/        */
    vline(x+i+1,y-1-l2-i,l2,105|128);
    /*            ____ *
     *           /___/ */
    hline(x+i+2,y-1-i,l2,105|128);
    /*           ____  *
     *           \___\ */
    hline(x+i+2,y+1+i,l2,95);
    /*         |\        *
     *         | |       *
     *          \|       */
    vline(x+i+1,y+2+i,l2,95|128);
    /*     |\            *
     *     | |           *
     *      \|           */
    vline(x-i-1,y-1-l2-i,l2,95|128);
    /* ____              *
     * \___\             */
    hline(x-i-1-l2,y-1-i,l2,95);
    /*  ____             *
     * /___/             */
    hline(x-i-1-l2,y+1+i,l2,105|128);
   /*       /|       *
     *     | |       *
     *     |/        */
    vline(x-i-1,y+2+i,l2,105|128);
  }
}

/* let's go! */
void main(void) {
  char i,j;

  *(char *)657=128;   // block chaset switching ( shift + C= )
  *(char *)53272U=21; // set PETSCII char set
  cls();

  while (1) {

    /* take care the star does not leave the screen buffer boundary,
       there is no exception handling for that, it will destroy memory! */
    for (j=0;j<5;++j) {
      // clear in first 2 frames required
      if (j<2) cls();
      // now draw star and wait for a new frame
      star(20,12,j);
      waitvsync();
      // got a new frame, switch over
      if (screenstart==0x400U) {
        *((char*)0xDD00)=*((char*)0xDD00)|0x02; // switch to $0400
        screenstart = 0x8400U;                  // draw to $8400
      } else {
        *((char*)0xDD00)=*((char*)0xDD00)&0xFD; // switch to $8400
        screenstart = 0x0400U;                  // draw to $0400
      }
      // set new color on 1st frame
      if (j==0) col(rand()&0x05);
      for (i=0;i<10;++i) waitvsync(); // wait 1/5s
    }
  }
}